package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.domain.ResultVo;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.mapper.UserMapper;
import com.zanxiang.sdk.common.util.RegisterUtil;
import com.zanxiang.sdk.domain.dto.UserDTO;
import com.zanxiang.sdk.domain.dto.UserTokenDTO;
import com.zanxiang.sdk.domain.params.UpdatePasswordParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.params.UserDetailParam;
import com.zanxiang.sdk.domain.vo.UserVO;
import com.zanxiang.sdk.service.UserService;
import com.zanxiang.sdk.service.UserTokenService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户信息
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    @Autowired
    private UserTokenService userTokenService;

    /**
     * 用户重置密码
     *
     * @param updatePasswordParam : 修改密码的参数
     * @param userData            : userData
     * @return : 返回修改结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVo<Boolean> updatePassword(UpdatePasswordParam updatePasswordParam, UserData userData) {
        String token = updatePasswordParam.getToken();
        String password = updatePasswordParam.getPassword();
        String username = updatePasswordParam.getUsername();
        //查询用户信息
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.checkPhone(username)) {
            queryWrapper.eq(User::getMobile, username);
        } else {
            queryWrapper.eq(User::getUsername, username);
        }
        User user = super.getOne(queryWrapper);
        if (user == null) {
            return new ResultVo<>(HttpStatusEnum.USERNAME_NOT_EXISTS);
        }
        //验证token是否存在
        HttpStatusEnum checkUserTokenEnum = userTokenService.checkUserToken(token, user.getId(), userData.getDeviceType());
        if (!Objects.equals(HttpStatusEnum.SUCCESS, checkUserTokenEnum)) {
            return new ResultVo<>(checkUserTokenEnum);
        }
        //密码校验
        HttpStatusEnum checkPasswordEnum = RegisterUtil.checkPassword(password);
        if (!Objects.equals(checkPasswordEnum, HttpStatusEnum.SUCCESS)) {
            return new ResultVo<>(checkPasswordEnum);
        }
        //修改用户密码
        super.update(new LambdaUpdateWrapper<User>()
                .set(User::getPassword, RegisterUtil.cmfPassword(password))
                .set(User::getUpdateTime, LocalDateTime.now())
                .eq(User::getUsername, username));
        //返回修改成功
        return new ResultVo<>(HttpStatusEnum.SUCCESS, Boolean.TRUE);
    }

    /**
     * 获取用户信息
     *
     * @param userDetailParam : 用户详情参数
     * @param userData        : userData
     * @return : 返回用户信息
     */
    @Override
    public ResultVo<UserVO> getUserDetail(UserDetailParam userDetailParam, UserData userData) {
        String token = userDetailParam.getToken();
        Integer deviceType = userData.getDeviceType();
        UserTokenDTO userTokenDTO = userTokenService.getTokenInfoByTokenDevice(token, deviceType);
        if (userTokenDTO == null) {
            return new ResultVo<>(HttpStatusEnum.INVALID_PARAMS);
        }
        //查询用户信息
        User user = super.getById(userTokenDTO.getUserId());
        if (user == null) {
            return new ResultVo<>(HttpStatusEnum.UNKNOWN_ERROR);
        }
        UserVO userVO = BeanUtils.copy(user, UserVO.class);
        return new ResultVo<>(HttpStatusEnum.SUCCESS, userVO);
    }

    /**
     * 根据手机号获取用户信息
     *
     * @param mobile : 用户手机号
     * @return : 返回用户信息
     */
    @Override
    public UserDTO getUserInfoByMobile(String mobile) {
        User user = super.getOne(new LambdaQueryWrapper<User>().eq(User::getMobile, mobile));
        return BeanUtils.copy(user, UserDTO.class);
    }

}
